function [rho,rho0,theta,tau] = rho_rrll(SAR,varargin)

% rho_rrll calculates complex correlation between SRR and SLL in polSAR data
% (orientation angle and helicity - if input is SLC we include multilooking)
%
% rho = rho_rrll(SHH,SHV,SVV);
% rho = rho_rrll(SHH,SHV,SVV,true);
% or
% [rho,rho0] = rho_rrll(SHH,SHV,SVV);
% [rho,rho0] = rho_rrll(SHH,SHV,SVV,true);
% or
% rho = rho_rrll(SAR);
% rho = rho_rrll(SAR,true);
% or 
% [rho,rho0] = rho_rrll(SAR);
% [rho,rho0] = rho_rrll(SAR,true);
% etc.
%
% If 2nd or 4th input parameter is 'true', plot intensity-hue of
% complex rho etc.
%
% Input either (1)
%   SAR   - covariance matrix representation, must be 3-D with
%             nine (9) (quad/full polarimetry) bands,
%             order is alphabetical in indexes with
%             real parts before imaginary parts (as read by
%             freadenvisar), for full polarimetry:
%               hhhh, re(hhhv), im(hhhv), re(hhvv), im(hhvv),
%               hvhv, re(hvvv), im(hvvv), vvvv
%
%   Covariance matrix representation
%
%   ShhShh* ShhShv* ShhSvv*     k    a   rho
%   ShvShh* ShvShv* ShvSvv*  =  a*   ksi b
%   SvvShh* SvvShv* SvvSvv*     rho* b*  zeta%
%
% or (2) 
%   SHH - SLC HH
%   SHV - SLC HV
%   SVV - SLC VV
%
% The processed SLC input is followed by multilooking
%
% Output
%   rho    - (complex) correlation between SRR and SLL
%   rho0   - (real) correlation between SRR and SLL for azimuthal symmetry
%   theta  - (real) first frame: orientation angle in degrees
%            second frame: 2nd derivative for the value theta
%            (must be positive for theta to represent minimum C22)
%             third frame: two times amplitude of C22
%   tau    - (real) helicity
%
% Look also at rho./rho0 (says the reference)
%
% See also rho_llrr, polsar_covmat_rotate_rowcol
%
% References
%
% J.-S. Lee and D. L. Schuler and T. L. Ainsworth and E. Krogager and
% D. Kasilingam and W.-M. Boerner
% "On the estimation of radar polarization orientation shifts induced by
% terrain slopes"
% IEEE Transactions on Geoscience and Remote Sensing 40(1), 30-41, Jan. 2002
% doi:10.1109/36.981347
%
% T. L. Ainsworth, D. L. Schuler and J.-S. Lee
% "Polarimetric SAR characterization of man-made structures in urban areas
% using normalized circular-pol correlation coefficients"
% Remote Sensing of Environment, 112, 2876-2885, 2008
% doi:10.1016/j.rse.2008.02.005
%
% M. Sugimoto, K. Ouchi and Y Nakamura
% "Four-Component Scattering Power Decomposition Algorithm with Rotation
% of Covariance Matrix Using ALOS-PALSAR Polarimetric Data",
% Remote Sensing 2012, 4, 2199-2209
% doi:10.3390/rs4082199
%
% Allan A. Nielsen
% "Rotation of Polarimetric SAR Covariance Matrices",
% IEEE International Geoscience and Remote Sensing Symposium, IGARSS 2023,
% Pasadena, California, USA, 16-21 July 2023.
% https://orbit.dtu.dk/en/publications/rotation-of-polarimetric-sar-covariance-matrices

% (c) Copyright 2021
% Allan Aasbjerg Nielsen, PhD, MSc
% alan@dtu.dk, https://people.compute.dtu.dk/alan
% 19 Mar 2021
% 12 Mar 2022 der2 added /AA

%%
if nargin<1 || nargin>4
    error('input is either three SLC input files: SHH, SHV, SVV or one multilooked covariance matrix')
end
if nargout>4
    error('no more than four outputs, rho, rho0, theta and tau')
end

%%
ihplot = false;
if nargin==2
    ihplot = true; %varargin{1};
elseif nargin==4
    ihplot = true; %varargin{3};
end

%%
if nargin==1 || nargin==2 % input is multilooked covariance matrix
    %%
    if ischar(SAR), SAR = flip(freadenvisar(SAR,'b')); end

    if ndims(SAR)~=3
        error('input must be 3-D');
    end

    %%
    ShhmSvv2 = 0.25*(SAR(:,:,1) - 2*SAR(:,:,4) + SAR(:,:,9));
    ShvShhmSvv = 0.5*(complex(SAR(:,:,2),-SAR(:,:,3)) - complex(SAR(:,:,7),SAR(:,:,8)));
    Shv2 = SAR(:,:,6);
%     clear SAR
    % numerator
    num = Shv2 - ShhmSvv2 + 2i*real(ShvShhmSvv);
    % denominator
    S2  = Shv2 + ShhmSvv2;
    aux = 2*imag(ShvShhmSvv);
    if nargout<2
        clear Shv2 ShhmSvv2 ShvShhmSvv
    end
    den = sqrt((S2 - aux).*(S2 + aux));
    clear aux
    rho = conj(num./den);

    %%
    if nargout>1 % rho0 for azimuthal symmetry
        % numerator
        num = Shv2 - ShhmSvv2;
        clear Shv2 ShhmSvv2
        % denominator
        den = S2; % sqrt((Shv2 + ShhmSvv2).*(Shv2 + ShhmSvv2));
        clear S2
        if nargout<3
            clear ShvShhmSvv
        end
        rho0 = num./den;    
        if nargout>2 % orientation angle \theta
%             theta = 0.25*atan2d(2*real(ShvShhmSvv),num);
            theta = 0.25*atan2d(2*sqrt(2)*(SAR(:,:,2)-SAR(:,:,7)), ...
                                SAR(:,:,1)-2*SAR(:,:,4)-2*SAR(:,:,6)+SAR(:,:,9));
%             % First derivative at theta, must be (very close to) zero
%             der1 = sind(4*theta).*( SAR(:,:,1) - 2*SAR(:,:,6) + SAR(:,:,9) - 2*SAR(:,:,4) ) ...
%                  - 2*sqrt(2)*cosd(4*theta).*( SAR(:,:,2) - SAR(:,:,7) );
            % Second derivative at theta (must be positive)
            der2 = 4*cosd(4*theta).*(SAR(:,:,1)-2*SAR(:,:,4)-2*SAR(:,:,6)+SAR(:,:,9)) ...
                + 8*sqrt(2)*sind(4*theta).*(SAR(:,:,2)-SAR(:,:,7));
            C22min = 0.25*cosd(4*theta).*( -SAR(:,:,1) + 2*SAR(:,:,6) - SAR(:,:,9) + 2*SAR(:,:,4) ) ...
                - sqrt(2)/2*sind(4*theta).*( SAR(:,:,2) - SAR(:,:,7) ) ...
                + 0.25*( SAR(:,:,1) + 2*SAR(:,:,6) + SAR(:,:,9) - 2*SAR(:,:,4) );
            C22max = 0.25*cosd(4*(theta+45)).*( -SAR(:,:,1) + 2*SAR(:,:,6) - SAR(:,:,9) + 2*SAR(:,:,4) ) ...
                - sqrt(2)/2*sind(4*(theta+45)).*( SAR(:,:,2) - SAR(:,:,7) ) ...
                + 0.25*( SAR(:,:,1) + 2*SAR(:,:,6) + SAR(:,:,9) - 2*SAR(:,:,4) );
            twoampl = abs(C22max-C22min);
            clear C22min C22max
            if min(der2(:))<0, imtool(der2>0); end
            theta = cat(3,theta,der2,twoampl);
            if nargout>3 % helicity \tau
                tau = 2*imag(ShvShhmSvv)./den;
            end
        end
    end

else % input is SLC

    warning('*** rho_rrll: theta wrong ***')

    %%
    Shh = SAR; % unelegant
    clear SAR
    Shv = varargin{1}; 
    Svv = varargin{2};
    clear varargin

    %% parameters for multilooking

    filtertype = 'gaussian';
    % filtersize = 5;
    % sigma = 1;
    % filtersize = 7;
    % sigma = 1.5; % 7x7
    filtersize = 9;
    sigma = 2; % 9x9
    % filtersize = 11;
    % sigma = 3; % 11x11

    % filtertype = 'average';
    % filtersize = 5;

    %%
    % if ~(ismatrix(Shh) || ismatrix(Shv) || ismatrix(Svv))
    %     error('input must be 2-D')
    % end
    if isreal(Shh) || isreal(Shv) || isreal(Svv)
        error('input must be complex')
    end
    [nr,nc] = size(Shh);
    [nr1,nc1] = size(Shv);
    if ~(nr==nr1 && nc==nc1)
        error('input must have same sizes')
    end
    [nr2,nc2] = size(Svv);
    if ~(nr==nr2 && nc==nc2)
        error('input must have same sizes')
    end

    %%
    aux = 0.5*(Shh - Svv);
    clear Shh Svv
    ShhmSvv2 = aux.*conj(aux);
    ShvShhmSvv = Shv.*conj(aux);
    clear aux
    Shv2 = Shv.*conj(Shv);
    clear Shv
    % numerator
    num = Shv2 - ShhmSvv2 + 2i*real(ShvShhmSvv);
    % denominator
    S2  = Shv2 + ShhmSvv2;
    aux = 2*imag(ShvShhmSvv);
    if nargout<2
        clear Shv2 ShhmSvv2 ShvShhmSvv
    end
    % den = sqrt((Shv2 + ShhmSvv2 - aux).*(Shv2 + ShhmSvv2 + aux));
    % multilook
    % if strcmp(filtertype,'gaussian')
    %	num = imgaussfilt(num,sigma); % requires real
        num = filter2(fspecial(filtertype,filtersize,sigma),num);
        S2  = filter2(fspecial(filtertype,filtersize,sigma),S2);
        aux = filter2(fspecial(filtertype,filtersize,sigma),aux);
    % end
    den = sqrt((S2-aux).*(S2+aux));
    clear aux
    rho = conj(num./den);

    %%
    if nargout>1 % rho0 for azimuthal symmetry
        % numerator
        num = Shv2 - ShhmSvv2;
        clear Shv2 ShhmSvv2
        % denominator
        % den = sqrt((Shv2 + ShhmSvv2).*(Shv2 + ShhmSvv2));
        % multilook
        % if strcmp(filtertype,'gaussian')
        %	num = imgaussfilt(num,sigma); % requires real
            num = filter2(fspecial(filtertype,filtersize,sigma),num);
        % end
        den = S2; % sqrt(S2.*S2);
        clear S2
        if nargout<3
            clear ShvShhmSvv
        end
        rho0 = num./den;
        if nargout>2 % orientation angle \theta
            ShvShhmSvv = filter2(fspecial(filtertype,filtersize,sigma),ShvShhmSvv);
            theta = 0.25*atan2d(2*real(ShvShhmSvv),num);
%             theta = 0.25*atan2d(2*sqrt(2)*(SAR(:,:,2)-SAR(:,:,7)), ...
%                                 SAR(:,:,1)-2*SAR(:,:,4)-2*SAR(:,:,6)+SAR(:,:,9));
            if nargout>3 % helicity \tau
                tau = 2*imag(ShvShhmSvv)./den;
            end
        end
    end

end

clear ShhmSvv2 Shv2 num den

%%
if ihplot

    % plot intensity-hue version of magnitude and phase of complex rho
    % rho = complex(medfilt2(real(rho),[3 3]),medfilt2(imag(rho),[3 3]));
    [nrows,ncols] = size(rho);
%     rho = conj(rho); % rho_llrr
    % amp = abs(rho);
    amp = rho.*conj(rho);
    phas = angle(rho);
    % intensity-hue/hue-value plot of amplitude and phase
    figure('Name','intensity-hue complex rho_rrll')
    % figure('Name','intensity-hue complex rho_llrr')
    imshow(hsv2rgb(cat(3,(phas+pi)/(2*pi),ones(nrows,ncols),amp)))
    colormap(gca,hsv(256)) % not needed for hsv2rgb but for colorbar
    cb = colorbar('SouthOutside');
    set(cb,'XTick',(0:64:256)/256)
    set(cb,'XTickLabel',{'-180','-90','0','90','180'})
    drawnow
    if nargout>1
        figure('Name','rho0')
        imshow(rho0,[-1 1])
%         colormap(gca,hsv(256)) % not needed for hsv2rgb but for colorbar
        cb = colorbar('SouthOutside');
%         set(cb,'XTick',(0:64:256)/256)
%         set(cb,'XTickLabel',{'-180','-90','0','90','180'})
        drawnow
        if nargout>2
            figure('Name','orientation angle 4*theta')
            imshow(4*theta(:,:,1),[-180 180])
            colormap(gca,hsv(256))
            cb = colorbar('SouthOutside');
            set(cb,'XTick',(-180:90:180))
            set(cb,'XTickLabel',{'-180','-90','0','90','180'})
            drawnow
            amp = 10*log10(twoampl);
%             amp = sqrt(twoampl);
%             amp = twoampl;
            ampmin = prctile(amp(:),0.5);
            ampmax = prctile(amp(:),99.5);
%             ampmin = min(amp(:));
%             ampmax = max(amp(:));
            amp = (amp-ampmin)/(ampmax-ampmin);
%             amp = amp.*amp;
            phas = 4*theta(:,:,1);
            % intensity-hue/hue-value plot of amplitude and phase
            figure('Name','intensity-hue (max(C22)-min(C22), 4*theta')
            imshow(hsv2rgb(cat(3,(phas+180)/360,ones(nrows,ncols),amp)))
            colormap(gca,hsv(256)) % not needed for hsv2rgb but for colorbar
            cb = colorbar('SouthOutside');
            set(cb,'XTick',(0:64:256)/256)
            set(cb,'XTickLabel',{'-180','-90','0','90','180'})
            drawnow            
            if nargout>3
                figure('Name','helicity tau')
                imshow(tau,[-1 1])
%                 colormap(gca,hsv(256)) % not needed for hsv2rgb but for colorbar
                cb = colorbar('SouthOutside');
%                 set(cb,'XTick',(0:64:256)/256)
%                 set(cb,'XTickLabel',{'-180','-90','0','90','180'})
                drawnow
            end
        end
    end

end
